#ifndef	_SYS_FS_INODE_H
#define	_SYS_FS_INODE_H

/* SVr4.0 1.16	*/

#include <sys/fcntl.h>
#include <sys/time.h>

#ifdef	__cplusplus
extern "C" {
#endif

/*
 * The I node is the focus of all local file activity in UNIX.
 * There is a unique inode allocated for each active file,
 * each current directory, each mounted-on file, each mapping,
 * and the root.  An inode is `named' by its dev/inumber pair.
 * Data in icommon is read in from permanent inode on volume.
 *
 * Each inode has 5 locks associated with it:
 *	i_rwlock:	Serializes write and setattr request
 *			and allows read requests to proceed in parallel.
 *			Serializes reads/updates to directories.
 *	i_contents:	Protects almost all of the fields in the inode
 *			except for those list below.
 *	i_tlock:	Protects just the i_atime, i_mtime, i_ctime,
 *			i_delayoff, i_delaylen, i_nextrio, i_writes
 *			and i_flag fields.
 *	ih_lock:	Protects inode hash chain buckets
 *	ifree_lock:	Protects inode freelist
 *
 * Lock ordering:
 *	i_rwlock > i_contents > ih_lock > i_tlock > ifree_lock
 *
 * SVR4 Extended Fundamental Type (EFT) support:
 * 	The inode structure has been enhanced to support
 *	32-bit user-id, 32-bit group-id, and 32-bit device number.
 *	Standard SVR4 ufs also supports 32-bit mode field.  For the reason
 *	of backward compatibility with the previous ufs disk format,
 *	32-bit mode field is not supported.
 *
 *	The current inode structure is 100% backward compatible with
 *	the previous inode structure if no user-id or group-id exceeds
 *	MAXUID (param.h), and no major or minor number of a device number
 *	stored in an inode exceeds 255.
 *
 */

#define	UID_Long  (int)65535
				/* flag value to indicate uid is 32-bit Long */
#define	GID_Long  (int)65535
				/* flag value to indicate gid is 32-bit Long */

#define	NDADDR	12		/* direct addresses in inode */
#define	NIADDR	3		/* indirect addresses in inode */
#define	FSL_SIZE (NDADDR+NIADDR-1) * sizeof (daddr_t)
				/* max fast symbolic name length is 56 */
struct 	icommon {
	short ic_smode;	/*  0: mode and type of file */
	short	ic_nlink;	/*  2: number of links to file */
	short	ic_suid;	/*  4: owner's user id */
	short	ic_sgid;	/*  6: owner's group id */
	int	ic_size;	/*  8: number of bytes in file */
	int	ic_size2;	/* 12: low order size of file, bytes */
	Long	ic_atime;	/* 16: time last accessed */
	Long	ic_atspare;
	Long	ic_mtime;	/* 24: time last modified */
	Long	ic_mtspare;
	Long	ic_ctime;	/* 32: last time inode changed */
	Long	ic_ctspare;
	daddr_t	ic_db[NDADDR];	/* 40: disk block addresses */
	daddr_t	ic_ib[NIADDR];	/* 88: indirect blocks */
	Long	ic_flags;	/* 100: status, currently unused */
	Long	ic_blocks;	/* 104: blocks actually held */
	Long	ic_gen;		/* 108: generation number */
	Long	ic_shadow;	/* 112: shadow inode */
	Long	ic_uid;		/* 116: Long EFT version of uid */
	Long	ic_gid;		/* 120: Long EFT version of gid */
	Long	ic_oeftflag;	/* 124: reserved */
};

/* kludge... define di_addr to be direct blk addr */
#define di_addr di_size
struct dinode {
	union {
		struct	icommon di_icom;
		char	di_size[128];
	} di_un;
};

#define	xxi_mode		i_ic.ic_smode
#define	xxi_nlink		i_ic.ic_nlink
#define	xxxi_uid	i_ic.ic_uid
#define	xxi_gid		i_ic.ic_gid
#define	xxi_smode		i_ic.ic_smode
#define	xxi_suid		i_ic.ic_suid
#define	xxi_sgid		i_ic.ic_sgid

#define direct dirent
#ifdef _LITTLE_ENDIAN
#define	xxi_size		i_ic.ic_size.val[0]
#define	xxi_size_o	i_ic.ic_size.val[1]
#else
#define	xxi_size		i_ic.ic_size.val[1]
#define	xxi_size_o	i_ic.ic_size.val[0]
#endif
#define	xxi_db		i_ic.ic_db
#define	xxi_ib		i_ic.ic_ib

#define	xxi_atime		i_ic.ic_atime
#define	xxi_mtime		i_ic.ic_mtime
#define	xxi_ctime		i_ic.ic_ctime

#define	xxi_shadow	i_ic.ic_shadow
#define	xxi_blocks	i_ic.ic_blocks
#ifdef _LITTLE_ENDIAN
/*
 * Originally done on x86, but carried on to all other little
 * architectures, which provides for file system compatibility.
 */
#define	xxi_ordev		i_ic.ic_db[1]	/* USL SVR4 compatibility */
#else
#define	xxi_ordev		i_ic.ic_db[0]	/* was i_oldrdev */
#endif
#define	xxi_gen		i_ic.ic_gen
#define	xxi_forw		i_chain[0]
#define	xxi_back		i_chain[1]

#if defined(DEBUG)
#define	xxi_cksum		i_ic.ic_oeftflag
#define	di_cksum	di_ic.ic_oeftflag
#endif /* DEBUG */

/* EFT transition aids - obsolete */
#define	oEFT_MAGIC	0x90909090
#define	di_oeftflag	di_ic.ic_oeftflag

#define	di_ic		di_un.di_icom
#define	di_mode		di_ic.ic_smode
#define	di_nlink	di_ic.ic_nlink
#define	di_uid		di_ic.ic_uid
#define	di_gid		di_ic.ic_gid
#define	di_smode	di_ic.ic_smode
#define	di_suid		di_ic.ic_suid
#define	di_sgid		di_ic.ic_sgid

#ifdef _LITTLE_ENDIAN
#define	xdi_size		di_ic.ic_size.val[0]
#else
#define	xdi_size		di_ic.ic_size.val[1]
#endif
#define	di_db		di_ic.ic_db
#define	di_ib		di_ic.ic_ib

#define	di_atime	di_ic.ic_atime
#define	di_mtime	di_ic.ic_mtime
#define	di_ctime	di_ic.ic_ctime

#ifdef _LITTLE_ENDIAN
#define	di_ordev	di_ic.ic_db[1]
#else
#define	di_ordev	di_ic.ic_db[0]
#endif
#define	di_shadow	di_ic.ic_shadow
#define	di_blocks	di_ic.ic_blocks
#define	di_gen		di_ic.ic_gen

/* flags */
#define	IUPD		0x0001		/* file has been modified */
#define	IACC		0x0002		/* inode access time to be updated */
#define	IMOD		0x0004		/* inode has been modified */
#define	ICHG		0x0008		/* inode has been changed */
#define	INOACC		0x0010		/* no access time update in getpage */
#define	IMODTIME	0x0020		/* mod time already set */
#define	IREF		0x0040		/* inode is being referenced */
#define	ISYNC		0x0080		/* do all allocation synchronously */
#define	IFASTSYMLNK	0x0100		/* fast symbolic link */
#define	IMODACC		0x0200		/* only access time changed; */
					/*   filesystem won't become active */
#define	IATTCHG		0x0400		/* only size/blocks have changed */
#define	IBDWRITE	0x0800		/* the inode has been scheduled for */
					/* write operation asynchrously */
#define	IBAD		0x1000		/* invalid inode contents */

/* modes */
#define	IFMT		0170000		/* type of file */
#define	IFIFO		0010000		/* named pipe (fifo) */
#define	IFCHR		0020000		/* character special */
#define	IFDIR		0040000		/* directory */
#define	IFBLK		0060000		/* block special */
#define	IFREG		0100000		/* regular */
#define	IFLNK		0120000		/* symbolic link */
#define	IFSHAD		0130000		/* shadow indode */
#define	IFSOCK		0140000		/* socket */

#define	ISUID		04000		/* set user id on execution */
#define	ISGID		02000		/* set group id on execution */
#define	ISVTX		01000		/* save swapped text even after use */
#define	IREAD		0400		/* read, write, execute permissions */
#define	IWRITE		0200
#define	IEXEC		0100

/* specify how the inode info is written in syncip() */
#define	I_SYNC		1		/* wait for the inode written to disk */
#define	I_DSYNC		2		/* wait for the inode written to disk */
					/* only if IATTCHG is set */
#define	I_ASYNC		0		/* don't wait for the inode written */

/* flags passed to itrunc(), indirtrunc(), and free() */
#define	I_FREE	0x00000001		/* inode is being freed */
#define	I_DIR	0x00000002		/* inode is a directory */
#define	I_IBLK	0x00000004		/* indirect block */
#define	I_CHEAP	0x00000008		/* cheap free */
#define	I_SHAD	0x00000010		/* inode is a shadow inode */
#define	I_QUOTA	0x00000020		/* quota file */

#ifdef	__cplusplus
}
#endif

#endif	/* _SYS_FS_INODE_H */
